﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;

namespace _4200Prototyper
{
    public partial class Form1 : Form
    {
        //Define serialport here so it is public throughout the rest of the code
        SerialPort serialPort = new SerialPort();
        
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //Start up the timer
            timer.Enabled = true;
            timer.Interval = Convert.ToInt32(txtInterval.Text);

            //Make the output text readonly but set the colours so that it looks normal (readonly looks ugly!)
            txtOutput.ReadOnly = true;
            txtOutput.ForeColor = Color.Black;
            txtOutput.BackColor = Color.White;

            //Set up serialport stuff
            serialPort.Parity = Parity.None;
            serialPort.DataBits = 8;
            serialPort.DataReceived += new SerialDataReceivedEventHandler(DataReceivedHandler);
        }

        //Allow the serialport to continuously read input from the arduino sketch
        private void DataReceivedHandler(object sender, SerialDataReceivedEventArgs e)
        {
            SerialPort sp = (SerialPort)sender;
            string indata = sp.ReadExisting();
            txtInput.AppendText(indata);
        }

        //Actually send a command to the serialport
        void SendCommand(TextBox textBox)
        {
            try
            {
                serialPort.Write(textBox.Text);
                txtOutput.AppendText(textBox.Text + Environment.NewLine);
            }
            catch
            {
                btnStopTimer.Enabled = false;
                btnStartTimer.Enabled = true;
                timer.Enabled = false;
                MessageBox.Show("There was a problem writing to the serial port. Timer has been stopped.", "ERROR");

            }

        }

        //Hold command - just changes the button text really
        void HoldCommand(TextBox textBox, Button button)
        {
            //If chkEnableHold is enabled, we want to constantly send the same command using the timer click event
            if (chkEnableHold.Checked == true)
            {
                if (button.Text == "Stop Hold")
                {
                    button.Text = "Hold";
                    button.ForeColor = Color.Black;
                }
                else
                {
                    button.Text = "Stop Hold";
                    button.ForeColor = Color.Red;
                }
            }
            //If chkEnableHold is disabled, we treat it like a normal SendCommand situation, but the MouseUp events on the buttons actually do something!
            else
                SendCommand(textBox);
        }

        //Function called upon MouseUp on hold buttons when chkEnableHold is false
        void ReleaseButton(TextBox textBox)
        {
            if(chkEnableHold.Checked == false)
            {
                SendCommand(textBox);
            }
        }

        //Check for holds everytime timer ticks
        private void timer_Tick(object sender, EventArgs e)
        {
            if (btnHold1.Text == "Stop Hold")
                SendCommand(txt1);
            else if (btnHold2.Text == "Stop Hold")
                SendCommand(txt2);
            else if (btnHold3.Text == "Stop Hold")
                SendCommand(txt3);
            else if (btnHold4.Text == "Stop Hold")
                SendCommand(txt4);
            else if (btnHold5.Text == "Stop Hold")
                SendCommand(txt5);
            else if (btnHold6.Text == "Stop Hold")
                SendCommand(txt6);
            else if (btnHold7.Text == "Stop Hold")
                SendCommand(txt7);
            else if (btnHold8.Text == "Stop Hold")
                SendCommand(txt8);
            else if (btnHold9.Text == "Stop Hold")
                SendCommand(txt9);
            else if (btnHold10.Text == "Stop Hold")
                SendCommand(txt10);
            else if (btnHold11.Text == "Stop Hold")
                SendCommand(txt11);
            else if (btnHold12.Text == "Stop Hold")
                SendCommand(txt12);
            else if (btnHold13.Text == "Stop Hold")
                SendCommand(txt13);
            else if (btnHold14.Text == "Stop Hold")
                SendCommand(txt14);
            else if (btnHold15.Text == "Stop Hold")
                SendCommand(txt15);
            else if (btnHold16.Text == "Stop Hold")
                SendCommand(txt16);
        }

        //Connect to the serialport
        private void btnConnect_Click(object sender, EventArgs e)
        {
            try
            {
                serialPort.BaudRate = 9600;
                serialPort.PortName = txtPort.Text;

                serialPort.Open();

                txtOutput.AppendText("SERIAL PORT OPENED" + Environment.NewLine);

                btnDisconnect.Enabled = true;
                btnConnect.Enabled = false;
            }
            catch
            {
                MessageBox.Show("There was an error connecting to the serial port", "Error");
            }
        }

        //Disconnect from the serialport
        private void btnDisconnect_Click(object sender, EventArgs e)
        {
            serialPort.Close();

            btnConnect.Enabled = true;
            btnDisconnect.Enabled = false;

            txtOutput.AppendText("SERIAL PORT CLOSED" + Environment.NewLine);
        }

        //Update the timer interval to slow down/speed up serial port output for Hold buttons
        private void btnUpdateInterval_Click(object sender, EventArgs e)
        {
            int parsedValue;
            if (!int.TryParse(txtInterval.Text, out parsedValue))
            {
                MessageBox.Show("Interval has to be an integer value.", "Error");
                return;
            }

            timer.Interval = Convert.ToInt32(txtInterval.Text);
        }

        //Clear the output textbox and reset the Lines text
        private void btnClear_Click(object sender, EventArgs e)
        {
            lblLines.Text = "0";
            txtOutput.Text = "";
            txtOutput.AppendText("CLEARED" + Environment.NewLine);
            txtInput.Text = "";
            txtInput.AppendText("CLEARED" + Environment.NewLine);
        }

        //Update number of lines in output textbox
        private void txtOutput_TextChanged(object sender, EventArgs e)
        {
            try
            {
                var lines = txtOutput.Lines.Count();
                lines -= String.IsNullOrWhiteSpace(txtOutput.Lines.Last()) ? 1 : 0;
                lblLines.Text = Convert.ToString(lines);
            }
            catch
            {

            }
        }

        //Update number of lines in input textbox
        private void txtInput_TextChanged(object sender, EventArgs e)
        {
            try
            {
                var lines = txtInput.Lines.Count();
                lines -= String.IsNullOrWhiteSpace(txtInput.Lines.Last()) ? 1 : 0;
                lblInputLines.Text = Convert.ToString(lines);
            }
            catch 
            { 
            
            }
        }

        //Stop the timer manually
        private void btnStopTimer_Click(object sender, EventArgs e)
        {
            timer.Enabled = false;
            btnStopTimer.Enabled = false;
            btnStartTimer.Enabled = true;
        }

        //Start the timer manually
        private void btnStartTimer_Click(object sender, EventArgs e)
        {
            timer.Enabled = true;
            btnStartTimer.Enabled = false;
            btnStopTimer.Enabled = true;
        }

        //Sender buttons
        private void btnSend1_Click(object sender, EventArgs e)
        {
            SendCommand(txt1);
        }

        private void btnSend2_Click(object sender, EventArgs e)
        {
            SendCommand(txt2);
        }

        private void btnSend3_Click(object sender, EventArgs e)
        {
            SendCommand(txt3);
        }

        private void btnSend4_Click(object sender, EventArgs e)
        {
            SendCommand(txt4);
        }

        private void btnSend5_Click(object sender, EventArgs e)
        {
            SendCommand(txt5);
        }

        private void btnSend6_Click(object sender, EventArgs e)
        {
            SendCommand(txt6);
        }

        private void btnSend7_Click(object sender, EventArgs e)
        {
            SendCommand(txt7);
        }

        private void btnSend8_Click(object sender, EventArgs e)
        {
            SendCommand(txt8);
        }

        private void btnSend9_Click(object sender, EventArgs e)
        {
            SendCommand(txt9);
        }

        private void btnSend10_Click(object sender, EventArgs e)
        {
            SendCommand(txt10);
        }

        private void btnSend11_Click(object sender, EventArgs e)
        {
            SendCommand(txt11);
        }

        private void btnSend12_Click(object sender, EventArgs e)
        {
            SendCommand(txt12);
        }

        private void btnSend13_Click(object sender, EventArgs e)
        {
            SendCommand(txt13);
        }

        private void btnSend14_Click(object sender, EventArgs e)
        {
            SendCommand(txt14);
        }

        private void btnSend15_Click(object sender, EventArgs e)
        {
            SendCommand(txt15);
        }

        private void btnSend16_Click(object sender, EventArgs e)
        {
            SendCommand(txt16);
        }

        private void btnSend17_Click(object sender, EventArgs e)
        {
            SendCommand(txt17);
        }

        private void btnSend18_Click(object sender, EventArgs e)
        {
            SendCommand(txt18);
        }

        private void btnSend19_Click(object sender, EventArgs e)
        {
            SendCommand(txt19);
        }

        private void btnSend20_Click(object sender, EventArgs e)
        {
            SendCommand(txt20);
        }

        private void btnSend21_Click(object sender, EventArgs e)
        {
            SendCommand(txt21);
        }

        private void btnSend22_Click(object sender, EventArgs e)
        {
            SendCommand(txt22);
        }

        private void btnSend23_Click(object sender, EventArgs e)
        {
            SendCommand(txt23);
        }

        private void btnSend24_Click(object sender, EventArgs e)
        {
            SendCommand(txt24);
        }

        private void btnSend25_Click(object sender, EventArgs e)
        {
            SendCommand(txt25);
        }

        private void btnSend26_Click(object sender, EventArgs e)
        {
            SendCommand(txt26);
        }

        private void btnSend27_Click(object sender, EventArgs e)
        {
            SendCommand(txt27);
        }

        private void btnSend28_Click(object sender, EventArgs e)
        {
            SendCommand(txt28);
        }

        private void btnSend29_Click(object sender, EventArgs e)
        {
            SendCommand(txt29);
        }

        private void btnSend30_Click(object sender, EventArgs e)
        {
            SendCommand(txt30);
        }

        private void btnSend31_Click(object sender, EventArgs e)
        {
            SendCommand(txt31);
        }

        private void btnSend32_Click(object sender, EventArgs e)
        {
            SendCommand(txt32);
        }

        //Hold Buttons - I changed all these click functions to actually be MouseDown functions, so the names are a bit misleading...
        private void btnHold1_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt1, btnHold1);
        }

        private void btnHold2_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt2, btnHold2);
        }

        private void btnHold3_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt3, btnHold3);
        }

        private void btnHold4_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt4, btnHold4);
        }

        private void btnHold5_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt5, btnHold5);
        }

        private void btnHold6_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt6, btnHold6);
        }

        private void btnHold7_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt7, btnHold7);
        }

        private void btnHold8_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt8, btnHold8);
        }

        private void btnHold9_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt9, btnHold9);
        }

        private void btnHold10_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt10, btnHold10);
        }

        private void btnHold11_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt11, btnHold11);
        }

        private void btnHold12_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt12, btnHold12);
        }

        private void btnHold13_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt13, btnHold13);
        }

        private void btnHold14_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt14, btnHold14);
        }

        private void btnHold15_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt15, btnHold15);
        }

        private void btnHold16_Click(object sender, MouseEventArgs e)
        {
            HoldCommand(txt16, btnHold16);
        }


        private void btnHold17_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt17, btnHold17);
        }

        private void btnHold18_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt18, btnHold18);
        }

        private void btnHold19_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt19, btnHold19);
        }

        private void btnHold20_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt20, btnHold20);
        }

        private void btnHold21_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt21, btnHold21);
        }

        private void btnHold22_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt22, btnHold22);
        }

        private void btnHold23_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt23, btnHold23);
        }

        private void btnHold24_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt24, btnHold24);
        }

        private void btnHold25_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt25, btnHold25);
        }

        private void btnHold26_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt26, btnHold26);
        }

        private void btnHold27_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt27, btnHold27);
        }

        private void btnHold28_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt28, btnHold28);
        }

        private void btnHold29_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt29, btnHold29);
        }

        private void btnHold30_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt30, btnHold30);
        }

        private void btnHold31_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt31, btnHold31);
        }

        private void btnHold32_MouseDown(object sender, MouseEventArgs e)
        {
            HoldCommand(txt32, btnHold32);
        }

        //Mouse up
        private void btnHold1_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt1b);
        }

        private void btnHold2_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt2b);
        }

        private void btnHold3_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt3b);
        }

        private void btnHold4_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt4b);
        }

        private void btnHold5_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt5b);
        }

        private void btnHold6_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt6b);
        }

        private void btnHold7_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt7b);
        }

        private void btnHold8_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt8b);
        }

        private void btnHold9_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt9b);
        }

        private void btnHold10_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt10b);
        }

        private void btnHold11_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt11b);
        }

        private void btnHold12_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt12b);
        }

        private void btnHold13_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt13b);
        }

        private void btnHold14_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt14b);
        }

        private void btnHold15_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt15b);
        }

        private void btnHold16_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt16b);
        }

        private void btnHold17_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt17b);
        }

        private void btnHold18_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt18b);
        }

        private void btnHold19_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt19b);
        }

        private void btnHold20_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt20b);
        }

        private void btnHold21_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt21b);
        }

        private void btnHold22_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt22b);
        }

        private void btnHold23_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt23b);
        }

        private void btnHold24_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt24b);
        }

        private void btnHold25_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt25b);
        }

        private void btnHold26_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt26b);
        }

        private void btnHold27_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt27b);
        }

        private void btnHold28_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt28b);
        }

        private void btnHold29_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt29b);
        }

        private void btnHold30_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt30b);
        }

        private void btnHold31_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt31b);
        }

        private void btnHold32_MouseUp(object sender, MouseEventArgs e)
        {
            ReleaseButton(txt32b);
        }

    }
}
